from cProfile import Profile
from functools import wraps
import sys


def profile(console=False, sort="cumulative"):
    def decorator(fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            print(f"### Profiling {fn.__name__}", file=sys.stderr)
            profiler = Profile()
            result = None
            try:
                result = profiler.runcall(fn, *args, **kwargs)
            finally:
                filename = fn.__name__ + ".prof"
                if console:
                    profiler.print_stats(sort)
                    print(f"### {fn.__name__}", file=sys.stderr)
                else:
                    profiler.dump_stats(filename)
                    print(f"### Profile saved to {filename}", file=sys.stderr)
            return result
        return wrapper
    return decorator

